/**
 * @file cw32f002_flash.h
 * @author your name (you@domain.com)
 * @brief 
 * @version 0.1
 * @date 2021-05-13
 * 
 * @copyright Copyright (c) 2021
 * 
 */

/*******************************************************************************
*
* 代码许可和免责信息
* 武汉芯源半导体有限公司授予您使用所有编程代码示例的非专属的版权许可，您可以由此
* 生成根据您的特定需要而定制的相似功能。根据不能被排除的任何法定保证，武汉芯源半
* 导体有限公司及其程序开发商和供应商对程序或技术支持（如果有）不提供任何明示或暗
* 含的保证或条件，包括但不限于暗含的有关适销性、适用于某种特定用途和非侵权的保证
* 或条件。
* 无论何种情形，武汉芯源半导体有限公司及其程序开发商或供应商均不对下列各项负责，
* 即使被告知其发生的可能性时，也是如此：数据的丢失或损坏；直接的、特别的、附带的
* 或间接的损害，或任何后果性经济损害；或利润、业务、收入、商誉或预期可节省金额的
* 损失。
* 某些司法辖区不允许对直接的、附带的或后果性的损害有任何的排除或限制，因此某些或
* 全部上述排除或限制可能并不适用于您。
*
*******************************************************************************/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __CW32F002_FLASH_H__
#define __CW32F002_FLASH_H__
                   
#ifdef __cplusplus
 extern "C" {
#endif

/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/
#include "cw32f002.h"

/*****************************************************************************/
/* Global pre-processor symbols/macros ('#define')                           */
/*****************************************************************************/
//============================================================

/** @defgroup FLASH状态 
  * @{
  */ 
#define FLASH_FLAG_OK                           (0UL)
#define FLASH_FLAG_BSY                          (bv5)/*!<FLASHBusyflag*/
#define FLASH_FLAG_PGERR                        (bv4)/*!<FLASHProgramerrorflag*/
#define FLASH_FLAG_WRPRTERR                     (bv1)/*!<FLASHWriteprotectederrorflag*/
#define FLASH_FLAG_PCERR                        (bv0)/*!<FLASHPCerrorflag*/
#define FLASH_ERROR_ADDR                        0x80

/** @defgroup 读访问周期延时 
  * @{
  */ 
#define FLASH_Latency_1                         ((uint32_t)0x00000000)/*!<FLASHOneLatencycycle*/
#define FLASH_Latency_2                         ((uint32_t)0x00000001)/*!<FLASHTwoLatencycycles*/
#define FLASH_Latency_3                         ((uint32_t)0x00000002)/*!<FLASHTHREELatencycycles*/
#define IS_FLASH_LATENCY(LATENCY)               (((LATENCY)==FLASH_Latency_1)||\
                                                ((LATENCY) == FLASH_Latency_2) || \
                                                ((LATENCY) == FLASH_Latency_3)) 



/** @defgroup 读保护等级 
  * @{
  */
#define FLASH_RDLEVEL0                          ((uint16_t)0x00)
#define FLASH_RDLEVEL1                          ((uint16_t)0x01)
#define FLASH_RDLEVEL2                          ((uint16_t)0x02)
#define FLASH_RDLEVEL3                          ((uint16_t)0x03)
#define IS_FLASH_RDLEVEL(LEVEL)                 (((LEVEL)==FLASH_RDLEVEL0)||\
                                                ((LEVEL) == FLASH_RDLEVEL1) || \
                                                ((LEVEL) == FLASH_RDLEVEL2) || \
                                                ((LEVEL) == FLASH_RDLEVEL3))

/** @defgroup 中断标志位 
  * @{
  */
#define FLASH_IT_PROG                           (bv4)
#define FLASH_IT_PAGELOCK                       (bv1)
#define FLASH_IT_PC                             (bv0)
#define IS_FLASH_IT(IT)                         (((IT)&0xFFFFFFEC==0x0UL)&&((IT)!=0x0UL))
#define IS_FLASH_GET_IT(IT)                     (((IT)==FLASH_IT_PROG)||\
                                                ((IT) == FLASH_IT_PAGELOCK) || \
                                                ((IT) == FLASH_IT_PC)) 

 /** @defgroup FLASH页 
  * @{
  */ 
#define FLASH_WRProt_AllPages                   ((uint32_t)0x0000FFFF)/*!<WriteprotectionofallPages*/
#define IS_FLASH_PAGE_Number(PAGENUMBER)        (((PAGENUMBER)>=0)&&((PAGENUMBER)<=39))
#define IS_FLASH_ADDRESS(ADDRESS)               (((ADDRESS)>=0x00000000)&&((ADDRESS)<0x00005000))

//============================================================ 
//设置FLASH读访问周期
void FLASH_SetLatency( uint32_t FLASH_Latency );
//获取FLASH的读保护等级
uint8_t FLASH_GetReadOutLevel( void );
//设置FLASH的读保护等级
void FLASH_SetReadOutLevel( uint16_t RdLevel );
//解锁FLASH所有页面
void FLASH_UnlockAllPages(void);
//锁定FLASH所有页面
void FLASH_LockAllPages( void );
//解锁FLASH指定页面：按照页面号解锁，0~39
uint8_t FLASH_UnlockPage( uint8_t Page_Number );
//解锁FLASH指定页面：按照起始和终止地址解锁
uint8_t FLASH_UnlockPages( uint32_t StartAddr , uint32_t EndAddr );
//擦除FLASH指定页面：按照页面号擦除，0~39
uint8_t FLASH_ErasePage( uint8_t Page_Number );
//擦除FLASH指定页面：按照起始和终止地址擦除
uint8_t FLASH_ErasePages( uint32_t StartAddr , uint32_t EndAddr );
//指定地址开始写一定长度的数据，按照字节写
uint8_t FLASH_WirteBytes( uint32_t WriteAddr, uint8_t *pWrBuf , uint16_t WrByteCnt );
//锁定FLASH指定页面：按照页面号锁定，0~39
uint8_t FLASH_LockPage( uint8_t Page_Number );
//锁定FLASH指定页面：按照起始和终止地址锁定
uint8_t FLASH_LockPages( uint32_t StartAddr , uint32_t EndAddr );
//FLASH指定中断使能配置
void FLASH_ITConfig(uint32_t FLASH_IT, FunctionalState NewState);
//获取指定FLASH中断标志位
ITStatus FLASH_GetITStatus(uint32_t FLASH_IT);
//清除指定FLASH中断标志位
void FLASH_ClearITPendingBit(uint32_t FLASH_IT);
//获取FLASH模块当前状态//
uint32_t FLASH_GetStatus(void);

//============================================================

#ifdef __cplusplus
}
#endif

#endif
